use warnings;
use strict;

package Net::Domain::SMD::File;
use Log::Report   'net-domain-smd';

use MIME::Base64       qw/decode_base64/;
use XML::LibXML        ();
use POSIX              qw/mktime tzset/;
use XML::Compile::Util qw/type_of_node/;
use List::Util         qw/first/;

=chapter NAME
Net::Domain::SMD::File - Info from one SMD file

=chapter SYNOPSIS

  my $smd  = Net::Domain::SMD->new(%options);
  my $smdf = $smd->read($fn);

=chapter DESCRIPTION
Draft RFC L<http://tools.ietf.org/html/draft-lozano-tmch-func-spec-06>
SMD information is packaged.  This object represents one file in wrapped
format.

=chapter METHODS

=section Constructors

=c_method new OPTIONS

=requires filename FILENAME
=cut

sub new($%) { my ($class, %args) = @_; (bless {}, $class)->init(\%args) }
sub init($)
{   my ($self, $args) = @_;

    # Clean object construction is needed when we are going to write
    # SMD files... but we won't for now.
    $self;
}

=c_method fromFile FILENAME, OPTIONS

=cut

sub fromFile($%)
{   my ($class, $fn, %args) = @_;
    my $self   = $class->new(%args);

    $self->{NDSF_fn} = $fn;
    my $schemas = $args{schemas};

    open my($fh), '<:raw', $fn
        or fault "cannot read from smd file {fn}", fn => $fn;

    my $xml;
  LINE:
    while(<$fh>)
    {   next LINE if m/^#|^\s*$/;   # not yet permitted in those files
        if( m/^-{3,}BEGIN .* SMD/)
        {   my @smd;
            while(<$fh>)
            {   last if m/^-{3,}END .* SMD/;
                push @smd, $_;
            }
            $xml = \decode_base64(join '', @smd);
            next LINE;
        }

        # Only few of the fields are of interest: often better inside XML
        my ($label, $value) = split /\:\s+/;
        defined $value && length $value or next;
        $label = lc $label;
        $value =~ s/\s*$//s;
        if($label eq 'u-labels')
        {   $self->{NDSF_labels} = [split /\s*\,\s*/, $value];
        }
        elsif($label eq 'marks')  # trademark names?  Comma list?
        {   $self->{NDSF_marks}  =  [split /\s*\,\s*/, $value];
        }

    }

    if($xml && $schemas)
    {   my $root = $schemas->dataToXML($xml);
        $self->{NDSF_payload} = $root;
        my $type = type_of_node $root;
#warn $root;

        my $data = $schemas->reader($type)->($root);
        $self->{NDSF_data}  = $data;
    }

    $self;
}
    
#----------------
=section Attributes

=method filename

=method payload
Returns the M<XML::LibXML::Document> contained in the file.

=method labels
Returns a list of u-labels.

=method marks
Returns the marks named in the "Marks" field.  The name insinuates that
this field may contains multiple values, but the format of this field
is not described in the spec.

=cut

sub filename()  {shift->{NDSF_fn}}
sub payload()   {shift->{NDSF_payload}}
sub labels()    { @{shift->{NDSF_labels} || []} }
sub marks()     { @{shift->{NDSF_marks}  || []} }

sub _data()     {shift->{NDSF_data}}     # hidden
sub _mark()     {shift->_data->{mark}}     # hidden

#----------------
=section XML content access

The following methods try to be smart, to hide complexities in the XML
structure, simplifying the main program.

=method smdID

=method from
Returns the timestamp reflecting the 'notBefore' field.

=method fromTime
The value of M<from()> converted into seconds from epoc.

=method until
Returns the timestamp reflecting the 'notAfter' field.

=method untilTime
The value of M<until()> converted into seconds from epoc.

=cut

sub smdID()     {shift->_data->{smd_id}}
sub from()      {shift->_data->{smd_notBefore}}
sub until()     {shift->_data->{smd_notAfter}}
sub fromTime()  {my $s = shift; $s->date2time($s->from)}
sub untilTime() {my $s = shift; $s->date2time($s->until)}

sub issuer()
{   my $i = shift->_data->{smd_issuerInfo} or return;
    # remove smd namespace prefixes
    my %issuer;
    while(my($k, $v) = each %$i)
    {   $k =~ s/smd_//;
        $issuer{$k} = $v;
    }
    \%issuer;
}

=method courts
Returns a list of HASHes, each representing one court.
See F<examples/mark-templ.pl>

=example
  foreach my $court ($smd->courts)
  {   print $court->{courtName}, "\n";
      print Dumper $court;
  }
=cut

sub courts()  { @{shift->_mark->{court} || []} }

=method trademarks
Returns a list of HASHes, each representing one trademark.
See F<examples/mark-templ.pl>

=example
  foreach my $trademark ($smd->trademarks)
  {   print $trademark->{markName}, "\n";
      my $holders = $trademark->{holder} || [];
      foreach my $holder (@$holders)
      {  print "Holder: $holder->{name}\n";
      }
  }
=cut

sub trademarks()  { @{shift->_mark->{trademark} || []} }

=method treaties
Returns a list of HASHes, each representing one treaty or statute (one
element of type "treatyOrStatute".  See F<examples/mark-templ.pl>
=cut

sub treaties()  { @{shift->_mark->{treatyOrStatute} || []} }

=method date2time STRING
Converts a time string into a time stamp (since epoch).
=cut

sub date2time($)
{   my ($self, $date) = @_;
    # For now, I only support Zulu time: 2013-07-12T12:53:48.408Z
    $date =~ m/^([0-9]{4})\-([0-1]?[0-9])\-([0-3]?[0-9])T([0-2]?[0-9])\:([0-5]?[0-9])\:([0-6]?[0-9])(\.[0-9]+)?Z?$/ or return;

    my $oldtz = $ENV{TZ};
    $ENV{TZ}  = 'UTC';
    tzset;

    my $sec = mktime $6, $5, $4, $3, $2-1, $1-1900;
    $sec += $7 if defined $7;

    $ENV{TZ} = $oldtz;
    tzset;

    $sec;
}

#---------------
=section Certificates

=method certificates OPTIONS
Returns the certificates as contained in the SMD file.

=option  issuer NAME
=default issuer <any issuer>
=cut

sub certificates(%)
{   my ($self, %args) = @_;

    my $tokens = $self->_data->{ds_Signature}{ds_KeyInfo}{__TOKENS} || [];
    my @certs  = map $_->certificate, @$tokens;

    my $issuer = $args{issuer};
    $issuer ? (grep $_->subject eq $issuer, @certs) : @certs;
}

1;
